export class Throttler {
    private hasBeenCalledWithinTime = false;
    private lastCallFunc: (() => Promise<any>) | null = null;
    private timeout: int | null = null;

    public constructor(private minimumTimeBetweenCall: number) {}

    public async call(func: () => any) {
        this.lastCallFunc = func;

        if (this.timeout) clearTimeout(this.timeout);
        this.timeout = setTimeout(() => {
            if (this.lastCallFunc) this.lastCallFunc();
        }, this.minimumTimeBetweenCall);

        if (this.hasBeenCalledWithinTime) return;

        this.hasBeenCalledWithinTime = true;
        await func();

        setTimeout(() => this.hasBeenCalledWithinTime = false, this.minimumTimeBetweenCall);
    }
}
